@@ -1,29 +1,31 @@
 import os
-import shutil
 import tempfile
 
+from io import open
 from subprocess import CalledProcessError
 
-from poetry.config import Config
-from poetry.utils.helpers import get_http_basic_auth
+from clikit.api.io import IO
+from clikit.io import NullIO
+
+from poetry.repositories.pool import Pool
+from poetry.utils._compat import encode
+from poetry.utils.env import Env
 from poetry.utils.helpers import safe_rmtree
 
+from .base_installer import BaseInstaller
+
 
 try:
     import urllib.parse as urlparse
 except ImportError:
     import urlparse
 
-from poetry.utils._compat import encode
-from poetry.utils.env import Env
-
-from .base_installer import BaseInstaller
-
 
 class PipInstaller(BaseInstaller):
-    def __init__(self, env, io):  # type: (Env, ...) -> None
+    def __init__(self, env, io, pool):  # type: (Env, IO, Pool) -> None
         self._env = env
         self._io = io
+        self._pool = pool
 
     def install(self, package, update=False):
         if package.source_type == "directory":
@@ -38,36 +40,40 @@ class PipInstaller(BaseInstaller):
 
         args = ["install", "--no-deps"]
 
-        if package.source_type == "legacy" and package.source_url:
+        if (
+            package.source_type not in {"git", "directory", "file", "url"}
+            and package.source_url
+        ):
+            repository = self._pool.repository(package.source_reference)
             parsed = urlparse.urlparse(package.source_url)
             if parsed.scheme == "http":
-                self._io.write_error(
+                self._io.error(
                     "    <warning>Installing from unsecure host: {}</warning>".format(
                         parsed.hostname
                     )
                 )
                 args += ["--trusted-host", parsed.hostname]
 
-            auth = get_http_basic_auth(
-                Config.create("auth.toml"), package.source_reference
-            )
-            if auth:
-                index_url = "{scheme}://{username}:{password}@{netloc}{path}".format(
-                    scheme=parsed.scheme,
-                    username=auth[0],
-                    password=auth[1],
-                    netloc=parsed.netloc,
-                    path=parsed.path,
-                )
-            else:
-                index_url = package.source_url
+            if repository.cert:
+                args += ["--cert", str(repository.cert)]
+
+            if repository.client_cert:
+                args += ["--client-cert", str(repository.client_cert)]
+
+            index_url = repository.authenticated_url
 
             args += ["--index-url", index_url]
+            if self._pool.has_default():
+                if repository.name != self._pool.repositories[0].name:
+                    args += [
+                        "--extra-index-url",
+                        self._pool.repositories[0].authenticated_url,
+                    ]
 
         if update:
             args.append("-U")
 
-        if package.hashes and not package.source_type:
+        if package.files and not package.source_url:
             # Format as a requirements.txt
             # We need to create a requirements.txt file
             # for each package in order to check hashes.
@@ -109,13 +115,14 @@ class PipInstaller(BaseInstaller):
             raise
 
     def run(self, *args, **kwargs):  # type: (...) -> str
-        return self._env.run("python", "-m", "pip", *args, **kwargs)
+        return self._env.run_pip(*args, **kwargs)
 
     def requirement(self, package, formatted=False):
         if formatted and not package.source_type:
             req = "{}=={}".format(package.name, package.version)
-            for h in package.hashes:
+            for f in package.files:
                 hash_type = "sha256"
+                h = f["hash"]
                 if ":" in h:
                     hash_type, h = h.split(":")
 
@@ -137,10 +144,18 @@ class PipInstaller(BaseInstaller):
             return req
 
         if package.source_type == "git":
-            return "git+{}@{}#egg={}".format(
+            req = "git+{}@{}#egg={}".format(
                 package.source_url, package.source_reference, package.name
             )
 
+            if package.develop:
+                req = ["-e", req]
+
+            return req
+
+        if package.source_type == "url":
+            return "{}#egg={}".format(package.source_url, package.name)
+
         return "{}=={}".format(package.name, package.version)
 
     def create_temporary_requirement(self, package):
@@ -156,9 +171,8 @@ class PipInstaller(BaseInstaller):
         return name
 
     def install_directory(self, package):
-        from poetry.io import NullIO
         from poetry.masonry.builder import SdistBuilder
-        from poetry.poetry import Poetry
+        from poetry.factory import Factory
         from poetry.utils._compat import decode
         from poetry.utils.env import NullEnv
         from poetry.utils.toml_file import TomlFile
@@ -191,7 +205,9 @@ class PipInstaller(BaseInstaller):
             # file since pip, as of this comment, does not support
             # build-system for editable packages
             # We also need it for non-PEP-517 packages
-            builder = SdistBuilder(Poetry.create(pyproject.parent), NullEnv(), NullIO())
+            builder = SdistBuilder(
+                Factory().create_poetry(pyproject.parent), NullEnv(), NullIO()
+            )
 
             with open(setup, "w", encoding="utf-8") as f:
                 f.write(decode(builder.build_setup()))
@@ -225,6 +241,6 @@ class PipInstaller(BaseInstaller):
         pkg = Package(package.name, package.version)
         pkg.source_type = "directory"
         pkg.source_url = str(src_dir)
-        pkg.develop = True
+        pkg.develop = package.develop
 
         self.install_directory(pkg)
